@@ -1,4 +1,4 @@
-from crocs.core import printable, RegexOperator, isword, notword
+from crocs.core import printable, RegexOperator, isword, notword, RegexStr
 from random import choice, randint
 
 class Any(RegexOperator):
@@ -66,6 +66,10 @@ class Word(RegexOperator):
     def to_regex(self):
         return r'\w'
 
+    def mkstmts(self, argrefs):
+        return "%s = %s()" % (self.instref(argrefs), 
+        self.__class__.__name__)
+
 class NotWord(RegexOperator):
     def invalid_data(self):
         return choice(isword) 
@@ -76,6 +80,10 @@ class NotWord(RegexOperator):
     def to_regex(self):
         return r'\W'
 
+    def mkstmts(self, argrefs):
+        return "%s = %s()" % (self.instref(argrefs), 
+        self.__class__.__name__)
+
 class Group(RegexOperator):
     """
     A normal group.
@@ -140,13 +148,35 @@ class GLink(RegexOperator):
         return self.to_regex()
 
     def invalid_data(self):
-        group = Group.grefs[elf.index]
+        group = Group.grefs[self.index]
         return group.invalid_data()
 
     def valid_data(self):
         group = Group.grefs[self.index]
         return group.valid_data()
 
+    def instref(self, argrefs):
+        """
+        The group link lhs should be the same
+        for the corresponding group, when serializing yregex to code 
+        
+        """
+
+        group = Group.grefs[self.index]
+        
+        # Save the group attribute name in the context of the
+        # yregex code.
+        instname = argrefs[group]
+        argrefs[self] = instname
+        return instname
+
+    def mkstmts(self, argrefs=dict()):
+        name = self.instref(argrefs)
+
+        # Shouldn't declerate statement. GLink is merely
+        # a wrapper to be used when parsing raw regex.
+        return ''
+
     def hasop(self, instance):
         return False
 
@@ -161,24 +191,36 @@ class NamedGroup(Group):
     (?P<name>...)
     """
 
-    def __init__(self, name, *args):
+    def __init__(self, gname, *args):
         super(NamedGroup, self).__init__(*args)
-        self.name  = name
+        self.gname  = gname
 
     def compile(self):
-        self.grefs[self.name] = self
+        self.grefs[self.gname] = self
 
-        self.data = r'(?P<%s>%s)' % (self.name, 
+        self.data = r'(?P<%s>%s)' % (self.gname, 
         super(Group, self).to_regex())
 
         self.compiled = True
-        self.map      = r'(?P=%s)' % self.name
+        self.map      = r'(?P=%s)' % self.gname
 
         self.input    = map(lambda ind: ind.valid_data(), self.args)
         self.input    = ''.join(self.input)
 
         return self.data
 
+    def mkstmts(self, argrefs):
+        name = self.instref(argrefs)
+        lines = super(NamedGroup, self).mkargs_stmts(argrefs)
+
+        args = ', '.join((argrefs[ind] for ind in self.args))
+        stmt = "%s = %s('%s', %s)"
+        stmt = stmt % (name, self.__class__.__name__, self.gname, args)
+        lines.append(stmt)
+
+        code = '\n'.join(lines)
+        return '%s\n%s' % (code, stmt)
+
 class Repeat(RegexOperator):
     """
     """
@@ -189,6 +231,7 @@ class Repeat(RegexOperator):
         super(Repeat, self).__init__(regex)
         self.min = min
         self.max = max
+        self.wrap = wrap
         self.greedy = greedy
 
         if isinstance(regex, str) and len(regex) > 1:
@@ -224,8 +267,20 @@ class Repeat(RegexOperator):
 
         return '%s{%s,%s}%s' % (regex, self.min, self.max, sym)
 
+    def mkstmts(self, argrefs):
+        name = self.instref(argrefs)
+
+        code = self.args[0].mkstmts(argrefs)
+        stmt = "%s = %s(%s, min=%s, max=%s, wrap=%s, greedy=%s)"
+
+        stmt = stmt % (name, 
+        self.__class__.__name__, argrefs[self.args[0]], self.min, 
+        self.max if self.max else "''", self.wrap, self.greedy)
+
+        return '%s\n%s' % (code, stmt)
+
 class ZeroOrMore(Repeat):
-    def __init__(self, regex, wrap=False, greedy=False):
+    def __init__(self, regex, min=0, max='', wrap=False, greedy=False):
         super(ZeroOrMore, self).__init__(regex, 0, wrap=wrap, greedy=greedy)
 
     def to_regex(self):
@@ -233,16 +288,16 @@ class ZeroOrMore(Repeat):
         '?' if self.greedy else '')
 
 class OneOrMore(Repeat):
-    def __init__(self, regex, wrap=False, greedy=False):
-        super(OneOrMore, self).__init__(regex, 1, wrap=wrap, greedy=greedy)
+    def __init__(self, regex, min=1, max='', wrap=False, greedy=False):
+        super(OneOrMore, self).__init__(regex, min, max, wrap=wrap, greedy=greedy)
 
     def to_regex(self):
         return '%s+%s' % (self.args[0].to_regex(), 
         '?' if self.greedy else '')
 
 class OneOrZero(Repeat):
-    def __init__(self, regex, wrap=False, greedy=False):
-        super(OneOrZero, self).__init__(regex, 0, 1, wrap=wrap, greedy=greedy)
+    def __init__(self, regex, min=0, max=1, wrap=False, greedy=False):
+        super(OneOrZero, self).__init__(regex, min, max, wrap=wrap, greedy=greedy)
 
     def to_regex(self):
         return '%s?%s' % (self.args[0].to_regex(), 
@@ -284,7 +339,21 @@ class ConsumeNext(RegexOperator):
         fmt = '(?<=%s)%s' if not self.neg else '(?<!%s)%s'
         return fmt % (self.args[0].to_regex(), self.args[1].to_regex())
 
-class ConsumeBack(RegexOperator):
+
+    def mkstmts(self, argrefs):
+        name = self.instref(argrefs)
+
+        code0 = self.args[0].mkstmts(argrefs)
+        code1 = self.args[1].mkstmts(argrefs)
+
+        stmt = "%s = %s(%s, %s, neg=%s)"
+
+        stmt = stmt % (name, self.__class__.__name__, 
+        argrefs[self.args[0]], argrefs[self.args[1]], self.neg)
+
+        return '%s\n%s\n%s' % (code0, code1, stmt)
+
+class ConsumeBack(ConsumeNext):
     """
     Lookahead assertion.
 
@@ -344,6 +413,10 @@ class Seq(RegexOperator):
     def clear(self):
         pass
 
+    def mkstmts(self, argrefs):
+        return "%s = %s('%s', '%s')" % (self.instref(argrefs), 
+        self.__class__.__name__, self.start, self.end)
+
 class Include(RegexOperator):
     """
     Sets.
@@ -441,3 +514,7 @@ class RegexComment(RegexOperator):
 
     def to_regex(self):
         return r'(?#%s)' % self.comment
+
+    def mkstmts(self, argrefs):
+        return "%s = %s('%s')" % (self.instref(argrefs), 
+        self.__class__.__name__, self.comment)
