@@ -36,12 +36,17 @@ It automatically escapes all strings that contain regex operators.
 
 It is necessary to escape yourself raw regex strings even inside character sets to make
 sure them are in fact equal to the resulting pythonic yregex that comes from xmake.
+
+The tests also check the serialization of the yregex structure to raw python code. The approach
+consists of using the BasicRegex.mkclone method and BasicRegex.mkcode method. The structure
+is serialized to raw code and executed then it is serialized again and tested against the initial 
+raw regex.
 """
 
 import unittest
 from crocs.regex import Include, Exclude, Any, OneOrZero, \
 OneOrMore, Group, ConsumeNext, ConsumeBack, X, Join, Seq, Repeat,\
-NamedGroup, ZeroOrMore
+NamedGroup, ZeroOrMore, Caret, Dollar
 from crocs.xparser import xmake
 from eacc.lexer import Lexer, LexError
 import re
@@ -55,9 +60,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
     
     def test1(self):
         expr0 = Include('x', 'y')
@@ -70,9 +76,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         expr0 = Include('x', 'y')
@@ -85,9 +92,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test3(self):
         expr0 = Include('x', 'y')
@@ -100,9 +108,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test4(self):
         expr0 = Include('%', '#')
@@ -115,8 +124,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test5(self):
         expr0 = Include('a', 'b')
@@ -127,9 +138,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test6(self):
         expr0 = Include('a', 'b')
@@ -140,9 +152,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test7(self):
         expr0 = Include(Seq('a', 'z'))
@@ -155,9 +168,10 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test8(self):
         expr0 = Include(Seq('a', 'z'))
@@ -172,89 +186,103 @@ class TestInclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test9(self):
         regstr = r'[a-z]+[0-9a-z]?(abc)\1'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test10(self):
         regstr = r'([a-z]+[0-9a-z])?(abc)'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test11(self):
         regstr = r'(([a-z]+[0-9a-z])(abc)\2)\2emm\3'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test12(self):
         regstr = r'[a-z]?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test13(self):
         regstr = r'([a-z]0)?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test14(self):
         regstr = r'(((((([0-9]+))))))\3\1\2'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test15(self):
         regstr = r'(([0-9]x)?([a-z]y))?mnc([a-z\&%])+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test16(self):
         regstr = r'([a-z\&\*\$])+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test17(self):
         regstr = r'[a-z\*]'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test18(self):
         regstr = r'(((((([0-9]))\5))))\3\1\2'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestExclude(unittest.TestCase):
     def test0(self):
@@ -266,6 +294,9 @@ class TestExclude(unittest.TestCase):
         yregex = xmake(regstr)
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test1(self):
         expr0 = Include(Seq('a', 'z'))
         expr1 = Exclude(Seq('1', '9'))
@@ -277,10 +308,12 @@ class TestExclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test2(self):
         expr0 = Exclude(Seq('a', 'z'))
         expr1 = OneOrMore(expr0)
@@ -295,9 +328,11 @@ class TestExclude(unittest.TestCase):
 
         yregex.test()
 
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test3(self):
         expr0 = Exclude(Seq('a', 'z'))
         expr1 = OneOrMore(expr0)
@@ -310,6 +345,8 @@ class TestExclude(unittest.TestCase):
         yregex = xmake(regstr)
         yregex.test()
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test4(self):
         expr0 = Exclude(Seq('a', 'z'))
@@ -323,61 +360,72 @@ class TestExclude(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test5(self):
         regstr = r'[^0-9]+([^abcd]?x([^eeee])?\2)'
         yregex = xmake(regstr)
 
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test6(self):
         regstr = r'[^0-9]+([^abcd]?x([^ee])+\2)'
         yregex = xmake(regstr)
 
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test7(self):
         regstr = r'[^0-9]+([^abcd]?x([^\*\*])\2)'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test8(self):
         regstr = r'[abc\*]+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test9(self):
         regstr = r'[\[abc\*]'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test10(self):
         regstr = r'[\]abc\*]'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test11(self):
         regstr = r'([\]abc\*])+\**[^\*\&\&\[\]]+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
 
         self.assertEqual(yregex.mkregex(), regstr)
-
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestAny(unittest.TestCase):
     def test0(self):
@@ -390,8 +438,10 @@ class TestAny(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         expr0 = Exclude(Seq('0', '9'))
@@ -403,8 +453,10 @@ class TestAny(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         expr0 = Include(Seq('0', '9'))
@@ -416,8 +468,10 @@ class TestAny(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test3(self):
         expr0 = Include(Seq('0', '9'))
@@ -430,8 +484,10 @@ class TestAny(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test4(self):
         expr0 = Include(Seq('0', '9'))
@@ -444,8 +500,10 @@ class TestAny(unittest.TestCase):
         yregex = xmake(regstr)
     
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test5(self):
         expr0 = Include(Seq('0', '9'))
@@ -468,8 +526,10 @@ class TestAny(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestOneOrZero(unittest.TestCase):
     def test0(self):
@@ -482,8 +542,10 @@ class TestOneOrZero(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         expr0 = Exclude(Seq('a', 'b'))
@@ -494,8 +556,10 @@ class TestOneOrZero(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         expr1 = OneOrZero('fooo')
@@ -505,22 +569,28 @@ class TestOneOrZero(unittest.TestCase):
         regstr = expr2.mkregex()
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test3(self):
         regstr = r'a?b?c?\*?\&?[c-z\*\(\)\[\]]?c?(a?e?)\1?c'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test4(self):
         regstr = r'(aa{1,3}?)\1'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestOneOrMore(unittest.TestCase):
     def test0(self):
@@ -533,8 +603,10 @@ class TestOneOrMore(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         expr0 = Include(Seq('a', 'b'))
@@ -545,8 +617,10 @@ class TestOneOrMore(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         expr1 = OneOrMore('fooo')
@@ -560,15 +634,19 @@ class TestOneOrMore(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test3(self):
         regstr = r'a+c+b+?(c+e+?)+?\1?(\1?c)+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestGroup(unittest.TestCase):
     def test0(self):
@@ -583,8 +661,10 @@ class TestGroup(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         expr0 = Group(X(), 'a', 'b', Include('abc'))
@@ -599,8 +679,10 @@ class TestGroup(unittest.TestCase):
 
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         expr0 = Group('a')
@@ -612,9 +694,11 @@ class TestGroup(unittest.TestCase):
         yregex = xmake(regstr)
         expr3.test()
 
-        yregex.hits()
+        
         yregex.test()
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test3(self):
         expr0 = Group('abc')
@@ -628,16 +712,18 @@ class TestGroup(unittest.TestCase):
         # Eacc should be capable of reading back the 
         # serialized string.
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test4(self):
         expr0 = Group('ab')
         expr1 = Group(expr0, expr0)
         regstr = expr1.mkregex()
-        print(regstr)
         yregex = xmake(regstr)
-        print('yRegex:', yregex)
-        # yregex.test()
-        # self.assertEqual(yregex.mkregex(), regstr)
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestNamedGroup(unittest.TestCase):
     def test0(self):
@@ -648,8 +734,10 @@ class TestNamedGroup(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
         
     def test1(self):
         expr0 = NamedGroup('alpha', 'X', OneOrMore(Group('a', 'b')), 'B')
@@ -660,8 +748,10 @@ class TestNamedGroup(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         # Check if it works for nested named groups.
@@ -681,8 +771,10 @@ class TestNamedGroup(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test3(self):
         expr0 = NamedGroup('foobar', Repeat(Any('a', X(), 'b')))
@@ -692,8 +784,10 @@ class TestNamedGroup(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestRepeat(unittest.TestCase):
     def test0(self):
@@ -706,8 +800,10 @@ class TestRepeat(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         expr0 = NamedGroup('oooo', Repeat(Any('a', X(), 'b')))
@@ -722,86 +818,111 @@ class TestRepeat(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         regstr = r'a{1,2}'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test3(self):
         regstr = r'\*{1,2}?c*'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test4(self):
         regstr = r'\*{1,8}?\&+?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test5(self):
         regstr = r'(\*cd){1,8}?\&+?\1+?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test6(self):
         regstr = r'((\*cd){1,8}?\&)+?(\1?ecdeeec+)+?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test7(self):
         regstr = r'((\*cd{1,}){1,8}?\&)+?(\1?ec?d){4,4}?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test8(self):
         regstr = r'(((\*cd{1,}){1,8}?\&)+?){3,3}?(\1?ecd){4,4}?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test9(self):
         regstr = r'a{1,3}c{3,3}?e{0,2}?c+?e{0,4}?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test10(self):
         regstr = r'(a{1,3}c{3,3}?e{2,2}?c+?e{0,4}?)\1{1,2}?'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test11(self):
         regstr = r'((a{1,3}c{3,3}?e{2,3}?c+?e{1,4}?)\2{1,2}?)'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test12(self):
         regstr = r'a'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
 class TestZeroOrMore(unittest.TestCase):
     def test0(self):
         expr0 = NamedGroup('alpha', Repeat(Any('a', X())))
@@ -814,8 +935,11 @@ class TestZeroOrMore(unittest.TestCase):
         regstr = expr8.mkregex()
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
         
     def test1(self):
         expr0 = NamedGroup('a999', Repeat('a'))
@@ -828,16 +952,22 @@ class TestZeroOrMore(unittest.TestCase):
         regstr = expr2.mkregex()
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test2(self):
         regstr = r'\*{1,3}(((a[0-9]c)\3{1,3}))((\$[a-z]\#)\2{1,3})\*{1,3}'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
 class TestConsumeNext(unittest.TestCase):
     def test0(self):
         expr0 = ConsumeNext(Group(X(), OneOrMore('alpha')), 
@@ -850,6 +980,9 @@ class TestConsumeNext(unittest.TestCase):
             yregex.test()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test1(self):
         expr0 = ConsumeNext(Group(X(), 'bar', X()), 
         Group(X(), OneOrMore('alpha')))
@@ -860,9 +993,11 @@ class TestConsumeNext(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test2(self):
         expr0 = ConsumeNext(Group(X(), 'bar', X()), 
         Group(X(), OneOrMore('alpha')))
@@ -877,9 +1012,42 @@ class TestConsumeNext(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test3(self):
+        regstr = '(?<=abc)((c+d)\2)e'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test4(self):
+        regstr = r'(?<=abc{3,3})((c+d{1,4})\2)e'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test5(self):
+        regstr = r'(?<!abc{3,3})((c+(d{1,4})eee+)\2)e'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+
 class TestConsumeBack(unittest.TestCase):
     def test0(self):
         expr0 = ConsumeBack(Group(X(), '1010101', X()), 
@@ -892,9 +1060,11 @@ class TestConsumeBack(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test1(self):
         expr0 = ConsumeBack(Group(OneOrMore(X()), 
         'aaa', X()), Group('aaa', X(), 'bbb'))
@@ -907,8 +1077,160 @@ class TestConsumeBack(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
+
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test2(self):
+        regstr = r'abc.+(?=eee)e'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test3(self):
+        regstr = r'(ac?c).+(?=e{1,3})e'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test4(self):
+        regstr = r'(ac{1,3}c)(aa)+(?=e{1,3})e'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test5(self):
+        regstr = r'(\*a\&{1,5}c)(aa)+(?=e{1,3}c+)'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test6(self):
+        regstr = r'(\*a\&{1,5}c)\1+(a.+a)+(?=e{1,3}c+)'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test7(self):
+        regstr = r'(\*a\&{1,5}c)\1+(a.+a)+(?=e{1,3}c+(ab+)\3{1,3})'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test8(self):
+        regstr = '(\*a\&{1,5})+\1+(a.+a)+(?=e{1,3}c+(ab+)\3{1,3})'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test9(self):
+        regstr = r'(\*a\&{1,5})+\1+(a.+a)+(?=e{1,3}c+(ab+)\3{1,3})+'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test10(self):
+        regstr = r'(\*a\&{1,5})+\1+(a.+a)+(?!e{1,3}c+(ab+)\3{1,3})+'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test11(self):
+        regstr = r'abc(?!cde)'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test11(self):
+        regstr = r'abc(?!c(d+e)\1{1,3}0)'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test12(self):
+        regstr = r'abc(?=cde)cc'
+        yregex = xmake(regstr)
+
+        with self.assertRaises(AssertionError):
+            yregex.test()
+        
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test13(self):
+        # Possibly bug. It fails to match.
+        # regstr = r'Isaa.+c\ (?=Asimov)ee'
+        # But the regex below matches success to find a match.
+        regstr = r'Isaa.+c\ (?=Asimov).+'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
+    def test14(self):
+        # Possibly bug. It fails to match.
+        # regstr = r'Isaa.+c\ (?!Asimov)ee'
+        # But the regex below matches success to find a match.
+        regstr = r'Isaa.+c\ (?!Asimov).+'
+        yregex = xmake(regstr)
+
+        yregex.test()
+        self.assertEqual(yregex.mkregex(), regstr)
+
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestRegexComment(unittest.TestCase):
     def test0(self):
@@ -916,172 +1238,216 @@ class TestRegexComment(unittest.TestCase):
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test1(self):
         regstr = '[abc]*(?#aiosdu).+([ab]*)(?#asiodu\)[asd])'
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test2(self):
         regstr = '[a-z]*(?#aiosdu).+([0-9]*)(?#hehehe\)[abcde])'
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test3(self):
         regstr = '[a-z]*(?#aiosdu)(abc)+([0-9]+)(123)(?#....aaa\)[abcde])'
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test4(self):
         regstr = '[a-z]*(?#aiosdu)((ab)*)?([0-9]+)(123)(?#....aaa\)[abcde])\1aa'
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test4(self):
         regstr = r'[a-z](?#aiosdu)([0-9]+)\1sdius\1(123)(?#....aaa\)[abcde])\1aa'
 
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test5(self):
         regstr = 'a(?#aiosdu)*b'
         yregex = xmake(regstr)
 
         yregex.test()
-        yregex.hits()
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
 class TestNonCapture(unittest.TestCase):
     def test0(self):
         regstr = r'(?:ee)'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         regstr = '((?:.+)cc(ee.*))+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         regstr = '((?:.+)cc(ee.*))+\1+c\2+e'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         regstr = '((?:fooobar.+)cc(ee.*))+\1+c\2+e'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestWord(unittest.TestCase):
     def test0(self):
         regstr = r'ee\*@\w+hehe'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         regstr = r'(e\&e\*@\w*cc)+(?:cee\w+)uu?\1'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestNotWord(unittest.TestCase):
     def test0(self):
         regstr = r'foo\W+bar'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         regstr = r'(\*\W+bar)+(@\1+)\&\2{1,3}'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
 class TestCaret(unittest.TestCase):
     def test0(self):
         regstr = r'^eudof.c+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        print(yregex.mkcode())
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test1(self):
         regstr = r'^(\*\^ee)+'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
     def test2(self):
         regstr = r'^(\*\^ee)+c*\..'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
 
 class TestDollar(unittest.TestCase):
     def test0(self):
-        regstr = r'^eudof.c+$'
+        regstr = r'^((ab[a-z].c+))alpha(?#ajust)$'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test1(self):
         regstr = r'^(\*\^ee)+$'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test2(self):
         regstr = r'^(\*\^ee)+c*\..$'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
-
+        
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
     def test2(self):
         regstr = r'((\*\^ee)+c*\..)*$'
         yregex = xmake(regstr)
         yregex.test()
-        yregex.hits()
 
         self.assertEqual(yregex.mkregex(), regstr)
 
+        clone = yregex.mkclone()
+        self.assertEqual(clone.mkregex(), regstr)
+
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
